Module: TypedOperation::Operations::Lifecycle
- Included in:
- Base, ImmutableBase
- Defined in:
- lib/typed_operation/operations/lifecycle.rb
Overview
Hooks into Literal’s initialization lifecycle to call #prepare if defined.
Instance Method Summary collapse
-
#after_initialize ⇒ Object
This is called by Literal on initialization of underlying Struct/Data.
Instance Method Details
#after_initialize ⇒ Object
This is called by Literal on initialization of underlying Struct/Data. : () -> void
10 11 12 |
# File 'lib/typed_operation/operations/lifecycle.rb', line 10 def after_initialize prepare if respond_to?(:prepare) end |