Exception: TypedOperation::UnwrapError
- Inherits:
-
StandardError
- Object
- StandardError
- TypedOperation::UnwrapError
- Defined in:
- lib/typed_operation/result/failure.rb
Overview
Error raised when trying to unwrap a Failure result.
Error raised when trying to unwrap a Failure result.